/// <reference path="../../typings/index.d.ts" />

import * as sequelize from 'sequelize';


import {UserInterface, Address} from './user_types';

interface UserInstance extends sequelize.Instance<UserInterface>, UserInterface {}

interface UserModel extends sequelize.Model<UserInstance, UserInterface> {}

var UserInitialized: boolean = false;
export var User: UserModel;
export function initUser(seq: sequelize.Sequelize): void {
  if (UserInitialized) {
    return;
  }

  User = <UserModel>seq.define<UserInstance, UserInterface>('User', {
    'username': {
      type: sequelize.STRING,
      primaryKey: true
    },
    'password': {
      type: sequelize.TEXT('medium')
    },
    'address_state': {
      type: sequelize.STRING
    },
    'address_addressLine1': {
      type: sequelize.STRING
    },
    'address_addressLine2': {
      type: sequelize.STRING
    },
    'address_postalCode': {
      type: sequelize.INTEGER
    },
    'address': {
      type: sequelize.VIRTUAL,
      get: function(): Address {
        return {
          'state': this.get('address_state'),
          'addressLine1': this.get('address_addressLine1'),
          'addressLine2': this.get('address_addressLine2'),
          'postalCode': this.get('address_postalCode')
        };
      },
      set: function(val: Address) {
        this.setDataValue('address_state', val.state);
        this.setDataValue('address_addressLine1', val.addressLine1);
        this.setDataValue('address_addressLine2', val.addressLine2);
        this.setDataValue('address_postalCode', val.postalCode);
      }
    },
    'signupDate': {
      type: sequelize.DATE
    }
  }, {  });

  UserInitialized = true;
}

export function init(seq: sequelize.Sequelize): void {
  initUser(seq);
};